@@ -86,6 +86,7 @@ extern void pathdefaults(pathptr, int, int);
 extern void instancedefaults(objinstptr, objectptr, int, int);
 extern void labeldefaults(labelptr, u_char, int, int);
 
+extern void converttocurve();
 extern void poly_add_point(polyptr, XPoint *);
 extern void drawdot(int, int);
 extern void copyalleparams(genericptr, genericptr);
@@ -129,7 +130,7 @@ extern void setpage(Boolean);
 extern int changepage(short);
 extern void newpage(short);
 extern void pushobject(objinstptr);
-extern void push_stack(pushlistptr *, objinstptr);
+extern void push_stack(pushlistptr *, objinstptr, char *);
 extern void pop_stack(pushlistptr *);
 extern void free_stack(pushlistptr *);
 extern void popobject(xcWidget, pointertype, caddr_t);
@@ -179,8 +180,10 @@ extern void elemrotate(genericptr *, short, XPoint *);
 extern void elementrotate(short, XPoint *);
 extern void edit(int, int);
 extern void pathedit(genericptr);
-extern void xc_lower(short *);
-extern void xc_raise(short *);
+extern void xc_lower();
+extern void xc_raise();
+extern void xc_top(short *, short *);
+extern void xc_bottom(short *, short *);
 extern void exchange(void);
 extern void elhflip(genericptr *, short);
 extern void elvflip(genericptr *, short);
@@ -304,6 +307,7 @@ Boolean library_object_unique(short, objectptr, objlistptr);
 void add_object_to_library(short, objectptr);
 u_char *find_delimiter(u_char *);
 char standard_delimiter_end(char);
+void output_graphic_data(FILE *, short *);
 
 extern Boolean CompareTechnology(objectptr, char *);
 extern TechPtr LookupTechnology(char *);
@@ -331,12 +335,12 @@ extern void savetechnology(char *, char *);
 extern void findfonts(objectptr, short *);
 extern void savefile(short);
 extern int printRGBvalues(char *, int, const char *);
-extern char *nosprint(char *);
+extern char *nosprint(char *, int *, int *);
 extern FILE *fileopen(char *, char *, char *, int);
 extern Boolean xc_tilde_expand(char *, int);
 extern Boolean xc_variable_expand(char *, int);
 extern short writelabel(FILE *, stringpart *, short *);
-extern char *writesegment(stringpart *, float *, int *);
+extern char *writesegment(stringpart *, float *, int *, int *, int *);
 extern int writelabelsegs(FILE *, short *, stringpart *);
 extern void printobjects(FILE *, objectptr, objectptr **, short *, int);
 extern void printrefobjects(FILE *, objectptr, objectptr **, short *);
@@ -697,6 +701,8 @@ extern void resolve_devnames(objectptr);
 extern void resolve_indices(objectptr, Boolean);
 extern void clear_indices(objectptr);
 extern void unnumber(objectptr);
+extern char *parsepininfo(objinstptr, char *, int);
+extern char *defaultpininfo(objinstptr, int);
 extern char *parseinfo(objectptr, objectptr, CalllistPtr, char *, char *, Boolean,
 		Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
@@ -806,6 +812,7 @@ extern short readcommand(short, FILE *);
 extern void UDrawGraphic(graphicptr);
 extern Imagedata *addnewimage(char *, int, int);
 extern graphicptr new_graphic(objinstptr, char *, int, int);
+extern graphicptr gradient_field(objinstptr, int, int, int, int);
 extern void invalidate_graphics(objectptr);
 extern void freegraphic(graphicptr);
 extern short *collect_graphics(short *);
@@ -904,9 +911,11 @@ extern void startselect(void);
 extern void trackselarea(void);
 extern void trackrescale(void);
 extern pointselect *addcycle(genericptr *, short, u_char);
+extern void addanticycle(pathptr, splineptr, short);
 extern void copycycles(pointselect **, pointselect **);
 extern void advancecycle(genericptr *, short);
 extern void removecycle(genericptr *);
+extern void removeothercycles(pathptr, genericptr);
 extern Boolean checkforcycles(short *, int);
 extern void makerefcycle(pointselect *, short);
 
@@ -951,6 +960,7 @@ extern int find_ordinal(int, labelptr, objinstptr);
 extern short UDrawChar(u_char, short, short, int, int, float);
 extern void UDrawString(labelptr, int, objinstptr);
 extern void UDrawStringNoX(labelptr, int, objinstptr);
+extern void CheckMarginStop(labelptr, objinstptr, Boolean);
 extern TextExtents ULength(labelptr, objinstptr, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
