@@ -158,6 +158,10 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
+   // Note:  Setting areawin->clipped to -1 prevents the clipmask from being
+   // applied to elements, 
+   areawin->clipped = -1;
+
    switch (ELEMENTTYPE(*(curobj->plist + instance))) {
       case ARC:
          UDrawArc((arcptr)elementptr, xobjs.pagelist[areawin->page]->wirewidth);
@@ -182,6 +186,7 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
          UDrawGraphic((graphicptr)elementptr);
 	 break;
    }
+   areawin->clipped = 0;
 }
 
 /*-------------------------------------------------*/
@@ -525,6 +530,60 @@ pointselect *addcycle(genericptr *pgen, short cycle, u_char flags)
    return pptr;
 }
 
+/*--------------------------------------------------------------*/
+/* If we edit the position of a control point, and the global	*/
+/* pathedit mode is set to TANGENTS, then we track the angle of	*/
+/* the adjoining curve, if there is one, by settings its cycle	*/
+/* flags to ANTIXY.						*/
+/*--------------------------------------------------------------*/
+
+void addanticycle(pathptr thispath, splineptr thisspline, short cycle)
+{
+   genericptr *ggen, *rgen;
+   splineptr otherspline;
+
+   if (areawin->pathedit == TANGENTS) {
+      for (ggen = thispath->plist; ggen < thispath->plist + thispath->parts;
+		ggen++)
+	 if (*ggen == (genericptr)thisspline) break;
+
+      if (*ggen != (genericptr)thisspline) return;	/* Something went wrong */
+
+      if (cycle == 1) {
+	 if (ggen > thispath->plist) {
+	    if (ELEMENTTYPE(*(ggen - 1)) == SPLINE) {
+	       addcycle(ggen - 1, 2, ANTIXY);
+	    }
+	 }
+	 else if (!(thispath->style & UNCLOSED)) {
+	    rgen = thispath->plist + thispath->parts - 1;
+	    if (ELEMENTTYPE(*rgen) == SPLINE) {
+	       otherspline = TOSPLINE(rgen);
+	       if (thisspline->ctrl[0].x == otherspline->ctrl[3].x &&
+			thisspline->ctrl[0].y == otherspline->ctrl[3].y)
+	          addcycle(rgen, 2, ANTIXY);
+	    }
+	 }
+      }
+      else if (cycle == 2) {	/* cycle should be only 1 or 2 */
+	 if (ggen < thispath->plist + thispath->parts - 1) {
+	    if (ELEMENTTYPE(*(ggen + 1)) == SPLINE) {
+	       addcycle(ggen + 1, 1, ANTIXY);
+	    }
+	 }
+	 else if (!(thispath->style & UNCLOSED)) {
+	    rgen = thispath->plist;
+	    if (ELEMENTTYPE(*rgen) == SPLINE) {
+	       otherspline = TOSPLINE(rgen);
+	       if (thisspline->ctrl[3].x == otherspline->ctrl[0].x &&
+			thisspline->ctrl[3].y == otherspline->ctrl[0].y)
+	          addcycle(rgen, 1, ANTIXY);
+	    }
+	 }
+      }
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Find the cycle numbered "cycle", and mark it as the		*/
 /* reference point.						*/
@@ -712,6 +771,20 @@ void removecycle(genericptr *pgen)
    *cycptr = NULL;
 }
 
+/*--------------------------------------*/
+/* Remove cycles from all parts of a	*/
+/* path other than the one passed	*/
+/*--------------------------------------*/
+
+void removeothercycles(pathptr ppath, genericptr pathpart)
+{
+   genericptr *pathgen;
+   for (pathgen = ppath->plist; pathgen < ppath->plist + ppath->parts;
+		pathgen++)
+      if (*pathgen != pathpart)
+	 removecycle(pathgen);
+}
+
 /*--------------------------------------*/
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
@@ -926,7 +999,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	case(GRAPHIC):
            /* check for graphic image center point inside area box */
 	   selected = test_insideness(TOGRAPHIC(curgen)->position.x,
-		TOGRAPHIC(curgen)->position.x, boxpts);
+		TOGRAPHIC(curgen)->position.y, boxpts);
 	   break;
 
         case(LABEL): {
@@ -955,7 +1028,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 
 	   tmpext = ULength(slab, areawin->topinstance, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
-			tmpext.width : tmpext.width >> 1) : 0);
+			tmpext.maxwidth : tmpext.maxwidth >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
 			tmpext.ascent : (tmpext.ascent + tmpext.base) >> 1)
 			: tmpext.base);
@@ -970,7 +1043,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
 	      tmpext = ULength(slab, areawin->topinstance, j + 1, NULL);
-	      tmpl2 = tmpext.width;
+	      tmpl2 = tmpext.maxwidth;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
 
@@ -1075,7 +1148,7 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
 void draw_all_selected()
 {
    int j;
-
+   
    if (areawin->hierstack != NULL) return;
 
    for (j = 0; j < areawin->selects; j++)
@@ -1088,27 +1161,19 @@ void draw_all_selected()
 
 void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
-   short *lastselect;
-   int i;
+   short saveselects;
 
    if (areawin->selects == 0) return;
    else if (areawin->hierstack != NULL) return;
 
-   /* reset the graphics state */
+   saveselects = areawin->selects;
 
+   /* reset the graphics state */
    SetFunction(dpy, areawin->gc, GXcopy);
 
-   for (i = 0; i < areawin->selects; i++) {
-      lastselect = areawin->selectlist + i;
-      XTopSetForeground(SELTOCOLOR(lastselect)); 
-
-      // Check is necessary due to failure to remove newly-created items
-      // from selection list when they are canceled.
-
-      if (*lastselect < thisobj->parts) {
-         geneasydraw(*lastselect, SELTOCOLOR(lastselect), thisobj, thisinst);
-      }
-   }
+   areawin->selects = 0;
+   drawarea(NULL, NULL, NULL);
+   areawin->selects = saveselects;
 }
 
 /*----------------------------------------------------------------------*/
