@@ -431,7 +431,7 @@ void SVGDrawObject(objinstptr theinstance, short level, int passcolor, pushlistp
 
    UPushCTM();
 
-   if (stack) push_stack(stack, theinstance);
+   if (stack) push_stack(stack, theinstance, NULL);
    if (level != 0)
        UPreMultCTM(DCTM, theinstance->position, theinstance->scale,
 			theinstance->rotation);
@@ -617,7 +617,7 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    tmpext = ULength(drawlabel, localinst, 0, NULL);
 
    newpoint.x = (tmpjust & NOTLEFT ?
-       (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0);
+       (tmpjust & RIGHT ? -tmpext.maxwidth : -tmpext.maxwidth >> 1) : 0);
    newpoint.y = (tmpjust & NOTBOTTOM ?
        (tmpjust & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) >> 1)
 		: -tmpext.base);
