@@ -1076,6 +1076,7 @@ XCWindowData *create_new_window()
    newwindow->gctype = 0;
    newwindow->time_id = 0;
    newwindow->clipmask = (Pixmap)NULL;
+   newwindow->pbuf = (Pixmap)NULL;
    newwindow->cmgc = (GC)NULL;
    newwindow->clipped = 0;
    newwindow->vscale = 1;
@@ -1130,7 +1131,7 @@ void pre_initialize()
    xobjs.pagelist[0]->wirewidth = 2.0;
    xobjs.pagelist[0]->outscale = 1.0;
    xobjs.pagelist[0]->background.name = (char *)NULL;
-   xobjs.pagelist[0]->pmode = 0;
+   xobjs.pagelist[0]->pmode = 2;	/* set auto-fit ON by default */
    xobjs.pagelist[0]->orient = 0;
    xobjs.pagelist[0]->gridspace = DEFAULTGRIDSPACE;
    xobjs.pagelist[0]->snapspace = DEFAULTSNAPSPACE;
@@ -1454,17 +1455,6 @@ void post_initialize()
       dbuf = XCreatePixmap(dpy, areawin->window, areawin->width,
 		areawin->height, DefaultDepthOfScreen(xcScreen(areawin->area)));
 #endif
-   if (areawin->clipmask == (Pixmap)NULL)
-      areawin->clipmask = XCreatePixmap(dpy, areawin->window, areawin->width,
-		areawin->height, 1);
-
-   if (areawin->cmgc == (GC)NULL) {
-      XGCValues values;
-      values.foreground = 0;
-      values.background = 0;
-      areawin->cmgc = XCreateGC(dpy, areawin->clipmask,
-		GCForeground | GCBackground, &values);
-   }
 
 #ifdef TCL_WRAPPER
 
